/*

Humboldt University Berlin
Department of Computer Science
Laboratory for Artificial Intelligence

AT Humboldt97
-------------
champion of RoboCup'97 - modified version for RoboCup'98

Copyright 1997,1998 Hans Dieter Burkhard, Markus Hannebauer, Jan Wendler, Kay Schroeter
                        (All rights reserved)

You may copy and distribute this program freely as long as you retain this notice.
If you make any changes or have any comments we would appreciate a message.

This programm was the winner-team of RoboCup97 (Simulation League).
It has been modified to cope with the new Soccer Server in RoboCup'98.

Program was developed by Hans Dieter Burkhard, Markus Hannebauer, Jan Wendler,
Kay Schroeter, Ralf Kuehnel, Pascal Mueller-Gugenberger, Adriana Foremniak,
Heike Mueller, Derrick Hepp and Amin Coja-Oghlan.

*/

#include <stdio.h>
#include <ctype.h>
#include <stream.h>
#include "Configuration.h"

Configuration *globalConf;

Configuration::Configuration() {

/* Die ganze Datenstruktur mit default-Werten */
  static options serverOptionsTemp[] = {
    // Server-Parameter
    {"goal_width",          (void *)&goalWidth,           DOUBLE,  7.32},
    {"player_size",         (void *)&playerSize,          DOUBLE,  1.0},
    {"player_decay",        (void *)&playerDecay,         DOUBLE,  0.5},
    {"player_rand",         (void *)&playerRand,          DOUBLE,  0.1},
    {"player_weight",       (void *)&playerWeight,        DOUBLE,  60.0},
    {"player_speed_max",    (void *)&playerSpeedMax,      DOUBLE,  32.0},
    {"stamina_max",         (void *)&playerStaminaMax,    DOUBLE,  2500.0},
    {"stamina_inc_max",     (void *)&playerStaminaInc,    DOUBLE,  50.0},
    {"recover_dec_thr",     (void *)&playerRecoverDecThr, DOUBLE, 0.3},
    {"recover_min",         (void *)&playerRecoverMin,    DOUBLE,  0.1},
    {"recover_dec",         (void *)&playerRecoverDec,    DOUBLE,  0.05},
    {"effort_dec_thr",      (void *)&playerEffortDecThr,  DOUBLE,  0.4},
    {"effort_min",          (void *)&playerEffortMin,     DOUBLE,  0.1},
    {"effort_dec",          (void *)&playerEffortDec,     DOUBLE,  0.05},
    {"effort_inc_thr",      (void *)&playerEffortIncThr,  DOUBLE,  0.9},
    {"effort_inc",          (void *)&playerEffortInc,     DOUBLE,  0.05},
    {"ball_size",           (void *)&ballSize,            DOUBLE,  0.15},
    {"ball_decay",          (void *)&ballDecay,           DOUBLE,  0.8},
    {"ball_rand",           (void *)&ballRand,            DOUBLE,  0.2},
    {"ball_weight",         (void *)&ballWeight,          DOUBLE,  0.2},
    {"ball_speed_max",      (void *)&ballSpeedMax,        DOUBLE,  32.0},
    {"dash_power_rate",     (void *)&dashPowerRate,       DOUBLE,  0.1},
    {"kick_power_rate",     (void *)&kickPowerRate,       DOUBLE,  0.1},
    {"kickable_margin",     (void *)&kickMargin,          DOUBLE,  1.0},
    {"maxpower",            (void *)&maxPower,            DOUBLE,  100.0},
    {"minpower",            (void *)&minPower,            DOUBLE,  -30.0},
    {"maxmoment",           (void *)&maxMoment,           DOUBLE,  180.0},
    {"minmoment",           (void *)&minMoment,           DOUBLE,  -180.0},
    {"visible_angle",       (void *)&visibleAngleInDegree,DOUBLE,  90.0},
    {"wind_dir",            (void *)&windDirection,       DOUBLE,  0.0},
    {"wind_force",          (void *)&windForce,           DOUBLE,  10.0},
    {"wind_rand",           (void *)&windRand,            DOUBLE,  0.3},
    {"catchable_area_l",    (void *)&catchAreaLength,     DOUBLE,  2.0},
    {"catchable_area_w",    (void *)&catchAreaWidth,      DOUBLE,  1.0},
    {"catch_probability",   (void *)&catchProbability,    DOUBLE,  1.0},
    {"ckick_margin",        (void *)&cornerKickMargin,    DOUBLE,  1.0},
    {"port",                (void *)&portNumber,          INT,     6000},
    {"coach_port",          (void *)&coachPortNumber,     INT,     6001},
    {"half_time",           (void *)&halfTimeInSecs,      INT,     600},
    {"simulator_step",      (void *)&simulatorStep,       INT,     100},
    {"send_step",           (void *)&sendStep,            INT,     150},
    {"recv_step",           (void *)&receiveStep,         INT,     20},
    {"say_msg_size",        (void *)&sayMessageSize,      INT,     256},
    {"hear_max",            (void *)&hearCapacityMax,     INT,     2},
    {"hear_inc",            (void *)&hearCapacityInc,     INT,     1},
    {"hear_decay",          (void *)&hearCapacityDecay,   INT,     1},
    {"catch_ban_cycle",     (void *)&catchCycle,          INT,     5},
    {"use_offside",         (void *)&useOffSide,          ONOFF,   1},
    {"forbid_kick_off_offside", (void *)&forbidKickOffOffside, ONOFF, 1},
    {"audio_cut_dist",      (void *)&audioCutOffDistance, DOUBLE,  50.0},
    {"quantize_step",       (void *)&distanceQuantStep,   DOUBLE,  0.1},
    {"quantize_step_l",     (void *)&landmarksQuantStep,  DOUBLE,  0.01},
    {"record_log",          (void *)&recLog,              BOOL,    1},
    {"verbose",             (void *)&verbose,             BOOL,    0},
    {"inertia_moment",      (void *)&inertiaMoment,       DOUBLE,  5.0},
    {"offside_active_area_size", (void *)&offsideArea,    DOUBLE,  9.15},
    
    // Ende
    {"\0",                          NULL,                0,       0}
  };

  serverOptions = serverOptionsTemp;

  int i;
  /* Einstellen der default-Werte */
  for(i=0; serverOptions[i].pointer != NULL; i++) {
    switch( serverOptions[i].type ) {
      case DOUBLE:
        *(double *)serverOptions[i].pointer = serverOptions[i].value;
        break;
      case INT:
        *(int *)serverOptions[i].pointer = (int)serverOptions[i].value;
        break;
      case BOOL:
      case ONOFF:
        *(bool *)serverOptions[i].pointer = (bool)serverOptions[i].value;
        break;
      default:
        break;
    }
  } // Ende for-Schleife

  calculateDependentParameters();

  staminaDashIgnoreThr=0;
  kickDirIgnoreOurPlayerDist=1000;
  kickDirIgnoreOurPlayerDir=90;
  kickDirIgnoreTheirPlayerDist=1000;
  kickDirIgnoreTheirPlayerDir=90;
}

char *Configuration::readLine( FILE *filePointer ) 
{
  static char buffer[MAX_MESSAGE];
  bool isDataAvailable = NO;
  int length = 0;
  do {
    if ( fgets( buffer + length, MAX_MESSAGE - length, filePointer ) == NULL) {
      if ( isDataAvailable == YES ) {
        break;
      } 
      else {
        return NULL;
      }
    }
    /* Kommentare und Leerzeilen ignorieren */
    if (buffer[length] == '#' || buffer[length] == '\n') {
      continue;
    }
    
    length = strlen( buffer );
    if ( length >= 2 && strcmp( buffer+length-2, "\\\n") == 0 ) {
      length -= 2;
      continue;
    }
    
    if ( length > 0 ) {
      isDataAvailable = YES;
    }
  } while ( isDataAvailable == NO );
  
  // Ersten ':' in ' ' umwandeln
  char *t = buffer;
  while(*t != '\0') {
    if (*t == ':') {
      *t = ' ' ;
      break;
    }
    t++;
  } 
   
  return buffer;
}
  

void Configuration::readConfigurationFile(char *configurationFile) {
  int i;
  FILE *filePointer;
  char *buffer;
  
  /* Oeffnen des Files */
  if ((filePointer = fopen(configurationFile,"r")) == NULL) {
    cout << "Can't open config file" << endl;
    return;
  }

  /* Einlesen der Parameter */

  while( ( buffer = readLine( filePointer ) ) != NULL) {
    int n;
    char parameterName[256];
    char onOff[16] ;

    n = sscanf(buffer,"%s", parameterName) ;
    if (n < 1) {
      cout << "Illegal line: (" << buffer << ")" << endl;
      continue ;
    }

    for(i = 0; serverOptions[i].pointer != NULL; i++) {
      if( strcmp(parameterName, serverOptions[i].optionName) ) {
        continue;
      }

      // Parameter gefunden
      switch( serverOptions[i].type ) {
        case DOUBLE:
          n = sscanf(buffer, "%s %lf", parameterName, 
              (double *)serverOptions[i].pointer) ;
          break ;
        case INT:
          n = sscanf(buffer, "%s %d", parameterName,
              (int *)serverOptions[i].pointer) ;
          break ;
        case BOOL:
          n = 2 ;
          *((int *)serverOptions[i].pointer) = YES ;
          break ;        
        case ONOFF:
          n = sscanf(buffer, "%s %s", parameterName, onOff) ;
          *((int *)serverOptions[i].pointer) = (!strcmp(onOff, "on")) ? YES : NO;
          break ;
      }

      if (n < 2) {
        cout << "Illegal line (" << parameterName << ")" << endl;
      }
      break ;
    }
    
    if (serverOptions[i].pointer == NULL)
      cout <<  "Illegal line (" << parameterName << ")" << endl;
  }

  fclose(filePointer);

  calculateDependentParameters();

  return;
}

void Configuration::calculateDependentParameters() {
  halfTime = halfTimeInSecs * 1000 / simulatorStep;
  visibleAngle = 3.1416/180.0 * visibleAngleInDegree;
  sendStepNorm = sendStep/4;

  kickRange = kickMargin + playerSize + ballSize;
}










