/*

Humboldt University Berlin
Department of Computer Science
Laboratory for Artificial Intelligence

AT Humboldt97
-------------
champion of RoboCup'97 - modified version for RoboCup'98

Copyright 1997,1998 Hans Dieter Burkhard, Markus Hannebauer, Jan Wendler, Kay Schroeter
                        (All rights reserved)

You may copy and distribute this program freely as long as you retain this notice.
If you make any changes or have any comments we would appreciate a message.

This programm was the winner-team of RoboCup97 (Simulation League).
It has been modified to cope with the new Soccer Server in RoboCup'98.

Program was developed by Hans Dieter Burkhard, Markus Hannebauer, Jan Wendler,
Kay Schroeter, Ralf Kuehnel, Pascal Mueller-Gugenberger, Adriana Foremniak,
Heike Mueller, Derrick Hepp and Amin Coja-Oghlan.

*/
#ifndef _CONFIGURATION_H_
#define _CONFIGURATION_H_

#define YES 1
#define NO  0
#define MAX_MESSAGE 4096

class Configuration
{
public:
  
  /** Konstruktor */
  Configuration();
  
  /** Liest eine Konfigurationsdatei ein. agent.conf und server.conf werden
  gleich behandelt. */
  void readConfigurationFile(char *configurationFile);
  
#define MAX_FILE_LEN 256

  /** @name Parameter des SoccerServers */
  //@{
    /// goal width
    double goalWidth;               
    /// player size
    double playerSize;              
    /// player decay
    double playerDecay;             
    /// player rand
    double playerRand;              
    /// player weight
    double playerWeight;            
    /// player speed max
    double playerSpeedMax;          
    /// player stamina max
    double playerStaminaMax;        
    /// player stamina inc
    double playerStaminaInc;        
    /// player recovery decriment threshold
    double playerRecoverDecThr;     
    /// player recovery min
    double playerRecoverMin;        
    /// player recovery decriment
    double playerRecoverDec;        
    /// player dash effort decriment threshold
    double playerEffortDecThr;      
    /// player dash effrot min
    double playerEffortMin;         
    /// player dash effort decriment
    double playerEffortDec;         
    /// player dash effort incriment threshold
    double playerEffortIncThr;      
    /// player dash effort incriment
    double playerEffortInc;         
    /// ball size
    double ballSize;                
    /// ball decay
    double ballDecay;               
    /// ball rand
    double ballRand;                
    /// ball weight
    double ballWeight;              
    /// ball speed max
    double ballSpeedMax;            
    /// dash power rate
    double dashPowerRate;           
    /// kick power rate
    double kickPowerRate;           
    /// kickable margin
    double kickMargin;                         
    /// max power
    double maxPower;                
    /// min power
    double minPower;                
    /// max moment
    double maxMoment;               
    /// min moment
    double minMoment;               
    /// visible angle
    double visibleAngleInDegree;    
    /// wind direction
    double windDirection;           
    /// wind force
    double windForce;               
    /// wind random
    double windRand;                
    /// goalie catchable area length
    double catchAreaLength;         
    /// goalie catchable area width
    double catchAreaWidth;          
    /// goalie catchable possibility
    double catchProbability;         
    /// corner kick margin
    double cornerKickMargin;        
    /// port number
    int portNumber;                 
    /// coach port number
    int coachPortNumber;            
    /// half time
    int halfTimeInSecs;             
    /// simulator step interval msec
    int simulatorStep;              
    /// udp send step interval msec
    int sendStep;                   
    /// udp recv step intercal msec
    int receiveStep;                
    /// string size of say message
    int sayMessageSize;             
    /// player hear_capacity_max
    int hearCapacityMax;            
    /// player hear_capacity_inc
    int hearCapacityInc;            
    /// player hear_capacity_decay
    int hearCapacityDecay;          
    /// goalie catch ban cycle
    int catchCycle;                 
    /// flag for using off side rule
    bool useOffSide;                
    /// flag for forbid kick off offside
    bool forbidKickOffOffside;      
    /// audio cut off distance
    double audioCutOffDistance;     
    /// quantize step of distance
    double distanceQuantStep;       
    /// quantize step of distance for landmark
    double landmarksQuantStep;      
    /// flag for log recording
    bool recLog;                              
    /// flag for verbose mode
    bool verbose;
    /// intertia moment for turn
    double inertiaMoment;
    /// offside active area size
    double offsideArea;
  //@}

  // double playerRecoverInit;       /* player recovery init */
  // double playerEffortInit;        /* player dash effort init */
  // double controlRadiusWidth;      /* (control radius) - (player size) */

        
  /** @name Abhaengige, automatisch berechnete Parameter */
  //@{
    /// (halfTimeInSecs * 1000 / simulatorStep)
    int halfTime;                   
    /// (radFromDegree(visibleAngleInDegree))
    double visibleAngle;            
    /// (sendStep /= 4)
    int sendStepNorm;               
    /// (kickMargin+playerSize+ballSize)
    double kickRange;               
  //@}

  /* Sinkt die Stamina unter diesen Wert, werden DASHs ignoriert
     Defaultwert ist 0; Der Wert wird in agent.c beim Einlesen der
     Agentenkonfiguration gesetzt */

  double staminaDashIgnoreThr;

  /* bei der Auswahl der KickDir werden eigene bzw. gegnerische Spieler,
     die weiter als angegeben entfernt sind bzw. von der eigenen Blickrichtung
     abweichen, ignorriert */
  double kickDirIgnoreOurPlayerDist;
  double kickDirIgnoreOurPlayerDir;
  double kickDirIgnoreTheirPlayerDist;
  double kickDirIgnoreTheirPlayerDir;

private:
  ///
  void calculateDependentParameters();

  /** Liest eine Zeile aus einer geoeffneten Datei.
    Dabei werden Zeilen, die mit einem Backslash (\) gefolgt von einem
    newline (\n) enden, aneinandergehaengt. Leerzeilen und Zeilen, die mit einem
    (#) beginnen, werden ignoriert. Die Maximallaenge einer Zeile ist 
    \Ref{MAX_MESSAGE}.
    @param filePointer Zeiger auf Dateistruktur, so wie sie z.B. von fopen
      geliefert wird.
    @return Zeiger auf einen Buffer, der die naechste Zeile enthaelt, oder NULL,
      wenn das Dateiende erreicht worden ist. */
  char *readLine( FILE *filePointer );


  ///
  typedef struct {
    char optionName[32];
    void *pointer;
    int type;
    double value;
  } options;
  
  ///
  options *serverOptions; 

#define INT               1
#define DOUBLE            2
#define BOOL              3
#define STRING            4
#define ONOFF             5
#define NONE              99
  
};

  
/** Global verfuegbares Configuration-Object */
extern Configuration *globalConf;

#endif




