////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////  alinkli.h - Klasse fuer eine doppelt verkettete Liste
/////  Diese Klasse ist nicht public domain!
/////  Sie unterliegt dem Urheberrecht und Kopierrecht von 
/////  fa. dreisoft c/o Holger Dreissig und Markus Hannebauer.
/////  Sie ist nur fuer die Nutzung im Praktikum im SoSe 97 freigegeben.


#ifndef _ALINKLIST_H_
#define _ALINKLIST_H_

#include "alinkel.h"
#include <math.h>
#include <stdlib.h>

template <class T> class ALinkList
{
protected:
	long int pos, count;
	ALinkElem<T> *cursor, *head, *tail;

public:
  ALinkList (); // Konstruktor
  ~ALinkList (); // Destruktor
  ALinkList & operator=(ALinkList<T> &x); // Zuweisungsoperator
  ALinkList & operator+=(ALinkList<T> &x); // Konkatenation

  void MakeEmpty (); // Destruktor fr statische Objekte

  int IsEmpty ();
  long int GetPos ();
  long int GetCount ();

  int OffRight ();
  int OffLeft ();

  long int NextCurs (long int i, long int pos, long int count);
  void SetPos (long int i);
  void Next ();
  void Previous ();
  void Last ();
  void First ();

  T *Retrieve (long int i);
  T *Value ();
  void ChangeVal (T *x);
  long int Find (T *x);

  void Insert (long int i, T *x);
  void InsertRight (T *x);
  void InsertLeft (T *x);
  T* DeletePos (long int i);
  T* Delete ();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////  alinkli.c - Implementierung einer Klasse fuer eine doppelt verkettete Liste

#define UNDEF -1

template <class T>
ALinkList<T>::ALinkList ()
{
	pos = 0;
	count = 0;
	cursor = NULL;
	head = NULL;
	tail = NULL;
}

template <class T>
ALinkList<T>::~ALinkList ()
{
	while (!IsEmpty ())
		DeletePos (1);
}

template <class T>
ALinkList<T> & ALinkList<T>::operator=(ALinkList<T> &x)
{
  if (&x!=this)
  {
		x.First ();
		while (!x.OffRight ())
		{
			InsertRight (x.Value ());
			x.Next ();
		}
  }
  return (*this);
}

template <class T>
ALinkList<T> & ALinkList<T>::operator+=(ALinkList<T> &x) // Konkatenation
{
  x.First ();
  while (!x.OffRight ())
  {
    Last ();
    InsertRight (x.Value ());
	x.Next ();
  }
  return (*this);
}

template <class T> void ALinkList<T>::MakeEmpty ()
{
T* tmp;
  while (!IsEmpty ())
  {
    tmp = DeletePos (1);
	if (tmp!=NULL)
	  delete tmp;
  }
}

template <class T> int ALinkList<T>::IsEmpty ()
{
	return (count==0);
}

template <class T> long int ALinkList<T>::GetPos ()
{
	return pos;
}

template <class T> long int ALinkList<T>::GetCount ()
{
	return count;
}

template <class T> int ALinkList<T>::OffRight ()
{
	return (count==0 || pos==count+1);
}

template <class T> int ALinkList<T>::OffLeft ()
{
	return (pos==0);
}

// Unterabschnitt LinkList::SetPos

template <class T> long int ALinkList<T>::NextCurs (long int i, long int pos, long int count)
{
long int deltapos;

  deltapos = abs (pos-i);
  if (count-i<i-1)
    if (count-i<deltapos)
      return (count);
    else
      return (pos);
  else
    if (i-1<deltapos)
      return (1);
    else
      return (pos);
}

template <class T> void ALinkList<T>::SetPos (long int i)
{
long int tmp;

  if (IsEmpty () || i>count+1)
    return;
  else
  {
    tmp = NextCurs (i, pos, count);
    if (i==0)
    {
      pos = 0;
      cursor = NULL;
    }
    else
      if (i==count+1)
      {
		pos = count+1;
		cursor = NULL;
      }
      else
		if (tmp==1)
		{
			pos = 1;
			cursor = head;
			while (pos<i)
			{
				pos++;
				cursor = cursor->right;
			}
		}
		else
			if (tmp==count)
			{
				pos = count;
				cursor = tail;
				while (pos>i)
				{
					pos--;
					cursor = cursor->left;
				}
			}
			else
				if (pos<=i)
				{
					while (pos<i)
					{
						pos++;
						cursor = cursor->right;
					}
				}
				else
				{
					while (pos>i)
					{
						pos--;
						cursor = cursor->left;
					}
				}
  }
}

// Ende Unterabschnitt LinkList::SetPos

template <class T> void ALinkList<T>::Next ()
{
  if (!IsEmpty () && !OffRight ())
  {
    if (OffLeft ())
      cursor = head;
    else
      cursor = cursor->right;
    pos++;
  }
}

template <class T> void ALinkList<T>::Previous ()
{
  if (!IsEmpty () && !OffLeft ())
  {
    if (OffRight ())
      cursor = tail;
    else
      cursor = cursor->left;
    pos--;
  }
}

template <class T> void ALinkList<T>::Last ()
{
  if (!IsEmpty ())
  {
    pos = count;
    cursor = tail;
  }
}

template <class T> void ALinkList <T>::First ()
{
  if (!IsEmpty ())
  {
    pos = 1;
    cursor = head;
  }
}

template <class T> T* ALinkList<T>::Retrieve (long int i)
{
ALinkElem<T> *tmpCurs, *cursPos;
long int tmpPos;

  if (!IsEmpty () && i<=count && i>0)
  {
    tmpCurs = cursor;
    tmpPos = pos;
    SetPos (i);
    cursPos = cursor;
    cursor = tmpCurs;
    pos = tmpPos;
    return (cursPos->val);
  }
  else
    return (NULL);
}

template <class T> T* ALinkList<T>::Value ()
{
  if (!IsEmpty () && !OffLeft () && !OffRight ())
    return (cursor->val);
  else
    return (NULL);
}

template <class T> void ALinkList<T>::ChangeVal (T* x)
{
  if (!IsEmpty () && !OffLeft () && !OffRight ())
    cursor->val = x;
}

template <class T> long int ALinkList<T>::Find (T* x)
{
  if (IsEmpty ())
    return (UNDEF);
  else
  {
    pos = 1;
    cursor = head;
    while (!(pos==count) && !(cursor->val==x))
    {
      pos++;
      cursor = cursor->right;
    }
    if (pos==count && !(cursor->val==x))
      return (UNDEF);
    else
      return (pos);
  }
}

template <class T> void ALinkList<T>::Insert (long int i, T* x)
{
  if (i<=count)
  {
    if (!IsEmpty ())
    {
      SetPos (i);
      if (i==0)
      {
		cursor = head;
		cursor->InsLeft (x);
		pos = 1;
		cursor = cursor->left;
		head = cursor;
      }
      else
      {
		cursor->InsRight (x);
		Next ();
		if (i==count)
			tail = cursor;
      }
    }
    else
    {
      cursor = new ALinkElem<T>;
      if (cursor!=NULL)
      {
		head = cursor;
		cursor->val = x;
		tail = head;
		pos = 1;
      }
    }
    count++;
  }
}

template <class T> void ALinkList<T>::InsertRight (T *x)
{
	Insert (pos, x);
}

template <class T> void ALinkList<T>::InsertLeft (T *x)
{
	Insert (pos-1, x);
}

template <class T> T* ALinkList<T>::DeletePos (long int i)
{
ALinkElem<T> *tmp;
T* value;

  if (!IsEmpty () && i>0 && i<=count)
  {
    SetPos (i);
    if (i>1)
    {
      Previous ();
      tmp = cursor->DelRight ();
      if (tmp == tail)
		tail = cursor;
	  value = tmp->GetVal ();
	  delete tmp;
      Next ();
    }
    else
    {
      if (count>1)
      {
		Next ();
		pos = 1;
		tmp = cursor->DelLeft ();
		value = tmp->GetVal ();
		delete tmp;
		head = cursor;
      }
      else
      {
		value = cursor->GetVal ();
		delete cursor;
		pos = 0;
		head = NULL;
		tail = NULL;
      }
    }
    count--;
  }
  return value;
}

template <class T> T* ALinkList<T>::Delete ()
{
  return DeletePos (pos);
}

#endif
