#include "comm.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stream.h>

////////////////////////////////////////////////////////////
//  comm.cpp -- Implementierung

Comm::Comm (char* host, int port)
{
struct hostent* host_ent;
struct in_addr* addr_ptr;
struct sockaddr_in cli_addr;
int sockfd, val;
unsigned int bufsize = 64000;

    m_nSocketfd = -1;
    m_bClosed = 1;
    m_bProtocol = 0;

    /////  Finden des Hosts  /////////
    if ((host_ent = (struct hostent*) gethostbyname (host))==NULL)
    {
		if (inet_addr (host)==-1)
		{
			cerr << "Fehler beim Auffinden der Adresse!\n";
			exit (-1);
		}
    }
    else
    {
		addr_ptr = (struct in_addr*) *host_ent->h_addr_list;
		host = inet_ntoa (*addr_ptr);
    }
    
    /////  Socket aufrufen  /////////
    if ((sockfd = socket (AF_INET, SOCK_DGRAM, 0))<0)
    {
	   cerr << "Fehler bei der Initialisierung des Sockets!\n";
		exit (-1);
    }
    setsockopt (sockfd, SOL_SOCKET, SO_RCVBUF,  (char*) &bufsize, sizeof (int));
    
    /////  Binden des Sockets an die ClientAdresse  /////////
    bzero ((char *) &cli_addr, sizeof (cli_addr));
    cli_addr.sin_family = AF_INET;
    cli_addr.sin_addr.s_addr = htonl (INADDR_ANY);
    cli_addr.sin_port = htons (0);
    if (bind (sockfd, (struct sockaddr *) &cli_addr, sizeof (cli_addr))<0)
    {
        cerr << "Fehler beim Binden der ClientAdresse!\n";
        exit (-1);
    }
    
    m_nSocketfd = sockfd;
    bzero ((char *) &m_sServ_addr, sizeof (m_sServ_addr));
    m_sServ_addr.sin_family = AF_INET;
    m_sServ_addr.sin_addr.s_addr = inet_addr (host);
    m_sServ_addr.sin_port = htons (port);

    m_bClosed = 0;
}

Comm::~Comm ()
{
    CloseConnection (); 
}

void Comm::CloseConnection ()
{
    if (!m_bClosed)
    { 
         close (m_nSocketfd);
         m_bClosed = 1;
    }
}

void Comm::SetProtocol (int bProtocol)
{
	m_bProtocol = bProtocol;
}

int Comm::SendMessage (char* buf)
{
int n;

	if (m_bProtocol)
		cout << "Send: " << buf << '\n';
	 n = strlen (buf);
    if (sendto (m_nSocketfd, buf, n, 0, (struct sockaddr *) &m_sServ_addr, sizeof (m_sServ_addr))!=n)
         return (SEND_FAILED);
    /////  Warten, bis Kommando ausgefuehrt worden ist  ////////////
    //usleep (100000);
    return (SEND_SUCCESS);
}

int Comm::WaitForMessage (char* buf, int size, int secs, int usecs)
{
int servlen, n;
fd_set fset;
struct timeval tval;
struct timeval* ptval;

   FD_ZERO (&fset);
   FD_SET (m_nSocketfd, &fset);
   if (secs==WAIT_FOREVER)
	   ptval = NULL;
	else
	{
		ptval = &tval;
	   tval.tv_sec = secs;
	   tval.tv_usec = usecs;
   }
   if ((n=select (m_nSocketfd+1, &fset, NULL, NULL, ptval))>0)
   {
	   servlen = sizeof (m_sServ_addr);
		n = recvfrom (m_nSocketfd, buf, size, 0, (struct sockaddr *) &m_sServ_addr, &servlen);
	   buf [n] = '\n';
	   if (m_bProtocol)
	   {
	   char mess [MAXMESG];
		   sscanf (buf, "%[^\n]", mess);
		   cout << "Receive: " << mess<< '\n';
	   }
	   if (n==0)
			return RECV_NOMESSAGE;
		else
			return RECV_SUCCESS;	
	}
	else
		if (n==-1 && errno==EINTR)
		{
			buf [0] = '\0';
			return RECV_INTERRUPT;
		}
		else
         return RECV_TIMEOUT;
}

int Comm::GetLastMessage (char* buf, int size)
{
char* message;
int res;

	message = new char [size];
	memset (message, 0, size*sizeof (char));
	res = WaitForMessage (message, size, WAIT_FOREVER, 0);
	if (res!=RECV_SUCCESS)
	{
		delete message;
		return RECV_NOMESSAGE;
	}
	while (res==RECV_SUCCESS)
	{
		strcpy (buf, message);
		res = WaitForMessage (message, size, 0, 0);
	}
	delete message;
	return RECV_SUCCESS;
}
